{% extends 'murano_base.html' %}
{% load i18n %}
{% block title %}{% trans "Add Application" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Add Application") %}
{% endblock page_header %}

{% block main %}
  {% include 'services/_wizard_create.html' %}
{% endblock %}
