{% extends 'horizon/common/_data_table.html' %}
{% load i18n %}

{% block table_caption %}
    <tr class='table_caption'>
      <th class='table_header' colspan='{{ columns|length }}'>
        {{ table.render_table_actions }}
      </th>
    </tr>
{% endblock table_caption %}
