{% load custom_filters %}

{% for hidden in form.hidden_fields %}
  {{ hidden }}
{% endfor %}
{% if form.non_field_errors %}
  <div class="alert alert-message alert-error">
    {{ wizard.form.non_field_errors }}
  </div>
{% endif %}
{% for field in form.visible_fields %}
  <div class="control-group form-field clearfix{% if field.errors %} error{% endif %}">
  {% if field|is_checkbox %}
      {{ field }}{{ field.label_tag }}
  {% else %}
      {{ field.label_tag }}
    {% if field.errors %}
      {% for error in field.errors %}
        <span class="help-inline">{{ error }}</span>
      {% endfor %}
    {% endif %}
    <span class="help-block">{{ field.help_text }}</span>
    <div class="input">
      {{ field }}
    </div>
  {% endif %}
  </div>
{% endfor %}
