{% load i18n %}

<h3>{% trans 'Description:' %}</h3>
<p>{% blocktrans %}<strong>Name:</strong> Set up for identifying a package.{% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Tags:</strong> Used for identifying and filtering packages.{% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Public:</strong> Defines whether or not a package can be used by other tenants. (Applies to package dependencies){% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Active:</strong> Allows to hide a package from the catalog. (Applies to package dependencies){% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Description:</strong> Allows adding additional information about a package.{% endblocktrans %}</p>
