{% load i18n %}
{% load url from future %}
{% load static %}
{% block page_header %}
  <div class='page-header'>
      <h1>
          <a href=" {% url 'horizon:murano:environments:index' %}">
              {% blocktrans %}Environments{% endblocktrans %}
          </a> > {% blocktrans %}Environment{% endblocktrans %}
          {{ environment_name|truncatechars:60 }}
      </h1>
  </div>
{% endblock %}
