{% load i18n %}
{% load url from future %}
{% block page_header %}
  <div class='page-header'>
      <h2>
          <a href=" {% url 'horizon:murano:environments:index' %}">
              {% blocktrans %}Environments{% endblocktrans %}
          </a> >
          <a href=" {% url 'horizon:murano:environments:services' service.environment_id %}">
              {% blocktrans %}Environment{% endblocktrans %} {{ environment_name|truncatechars_html:30 }}
          </a> > {% blocktrans%}Component{% endblocktrans %}  {{ service_name|truncatechars:30 }}

      </h2>
  </div>
{% endblock %}
