{% extends 'murano_base.html' %}
{% load i18n %}
{% load static %}
{% load compress %}
{% block title %}{% trans "Package Definitions" %}{% endblock %}

{% block css %}
  {{ block.super }}
  {% compress css %}
    <link rel="stylesheet" href="{% static 'muranodashboard/css/packages.css' %}">
  {% endcompress %}
{% endblock %}

{% block page_header %}
    {% include "horizon/common/_page_header.html" with title=_("Package Definitions") %}
{% endblock page_header %}
{% block main %}
    {{ table.render }}
{% endblock %}
