table#murano div.table_cell_wrapper div.inline-edit-form input {
    /*
     * overrides from form-control, to make inline edit consistent
     * with the rest of UI
     */
    display: block;
    height: 32px;
    font-size: 13px;
    line-height: 1.42857;
    color: #555;
    background-color: #fff;
    background-image: none;
    border: 1px solid #ccc;
    border-radius: 4px;
    -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
    box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
    -webkit-transition: border-color ease-in-out 0.15s, box-shadow ease-in-out 0.15s;
    -o-transition: border-color ease-in-out 0.15s, box-shadow ease-in-out 0.15s;
    transition: border-color ease-in-out 0.15s, box-shadow ease-in-out 0.15s;

    /* Additional styles, not present in form-control */
    margin: 7px 7px 0 7px;
    width: calc(100% - 14px);
}

table#murano div.table_cell_wrapper div.inline-edit-form input:focus {
    border-color: #66afe9;
    outline: 0;
    -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(102, 175, 233, 0.6);
    box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(102, 175, 233, 0.6);
}

.table_header.catalog {
  border-bottom: 2px solid #e5e5e5;
  padding-bottom:5px;
  margin-bottom: 10px;
}
.table_search {
  min-width: 265px;
}

.catalog .table_actions {
  width: 100%;
}

.scrollable-menu {
  height: auto;
  max-height: 200px;
  max-width: 250px;
  overflow-x: hidden;
}

.scrollable-menu a {
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;

}
.murano-dropdown-menu {
  margin-top: -15px;
  margin-left: 20px;
}

h3.heading_switcher {
  font-size: 18px;
}

.add_env {
  margin-left: auto;
  margin-right: auto;
}
.no_envs {
  color: slategray;
}

.link.reference {
  stroke-dasharray: 5,5;
}

h3.heading_switcher a.btn {
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.catalog .table_actions a, .catalog .table_actions button {
  margin-left: 0;
}

.catalog .row_actions button, .catalog .row_actions a {
  background: none;
  float: none;
  display: block;
  padding: 5px 10px;
  color: black;
  text-align: left;
  border-radius: 0;
  border: 0 none;
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
}

.catalog .dropdown-menu li > a:hover, .catalog .dropdown-menu .active > a, .catalog .dropdown-menu .active > a:hover {
  color: black;
  background-color: #CDCDCD;
}

.catalog .dropdown.open .dropdown-toggle {
  color: black;
  background: #e6e6e6;
}

.app-list .actions {
  border-top:1px #ccc solid;
  padding-top:15px;
  margin-top:15px;
}
.app-list .app-description {
  height: 135px;
}

.app-list .app-description h4 {
  margin-top: 0;
}

.btn-link {
  padding: 5px 0;
}
.centering {
  margin-left: auto;
  margin-right: auto;
  text-align: center;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
}

.draggable_app img {
  margin-bottom: 5px;
  display: block;
}

.draggable_app .well {
  margin-bottom: 0;
}

.well .app-actions-container {
    padding-right: 0;
}

.draggable_app .may_overflow {
  max-width: 85%;
}

@media (min-width: 1020px) {
  .draggable_app .well, .no_apps.well {
    min-height: 105px;
  }
}

.search-icon {
  line-height: 32px;
}

.draggable_app {
  font-size: 11px;
}

.draggable_app.col-xs-2 {
  padding-left: 5px;
  padding-right: 5px;
}

.carousel-inner .row {
  margin-left: 0;
  margin-right: 0;
}

.carousel {
  padding: 0 40px;
  margin-bottom: 20px;
  margin-top: 2px;
}

.carousel-control {
  position: absolute;
  top: 38px;
  left: -7px;
  bottom: 0;
  width: 30px;
  opacity: 1;
  filter: alpha(opacity=100);
  font-size: 20px;
  color: #666;
  text-align: center;
  text-shadow: none;
}

.carousel-control:hover:focus {
  color: #d93c27;
}

.carousel-control:hover {
  color: #d93c27;
  opacity: 1;
}

.carousel-control:focus {
  color: #666;
}

.carousel-control.left {
  background: inherit;
}

.carousel-control.right {
  background: inherit;
  right: 7px;
}

.category-title {
  float: left;
  line-height: 26px;
  margin-top: 20px;
  font-weight: normal;
}

.extra_title {
  line-height: 26px;
  font-weight: normal;
  font-size: 24px;
}

.draggable_app .well:hover {
  border-color: #d93c27;
  background: #fff;
}

#envAppsCategory.dropdown {
  min-width: 160px;
}

#envAppsCategory > button{
  margin-top: 16px;
  max-width:60%;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}


#envAppsFilter {
  margin-top: 15px;
}

div.drop_component {
  border: 2px #ccc dashed;
  background-color: #EBF3F9;
  width: 94%;
  margin: 0 auto -2px;
  border-radius: 5px;
  display: none;
}

div.drop_component.over {
  border-color: #3B88C3;
}

#no_apps_found_message {
  display: none;
}

h3.link {
  color: yellow;
  opacity: 0.5;
}
h3.link:hover {
  opacity: 1;
}
h3.link .caret {
  border-top: 4px solid yellow !important;
}
.app .ribbon {
    position: absolute;
    bottom: 0;
    left: 0;
}
.draggable_app .ribbon{
    position: absolute;
    bottom: -4px;
}
.well:hover {
    float: none;
    border: 1px solid #bbb;
    background: #eee;
    -o-transition: all .15s;
    -moz-transition: all .15s ease-out;
    -webkit-transition: all .15s ease-out;
    transition: all .15s ease-out;
}

.row.row-header {
  padding-left: 8px;
  padding-right: 8px;
}
.row.app-list {
  margin-top: 20px;
  margin-bottom:20px;
}
.row .filter-bar {
  position: absolute;
  right: 18px;
}

.app-list .app {
  position:relative;
  -moz-border-radius: 8px;
  -webkit-border-radius: 8px;
  border-radius: 8px;
}
.app-list .app > div {
  margin:0;
}
.app-list .app div.description {
  margin-left:0;
  height: 100%;
}
.app-list .app h4 {
  margin-bottom:0.25em;
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
}
.app-list .app p {
  max-height: 75px;
  overflow: hidden;
}

.app-list .app .app-actions {
  float: right;
  margin-top: 5px;
}
.app-actions div[class*='span'] {
  margin-left:0;
}
.header.dropdown {
  display:inline-block;
  margin-left: 5px;
}
.header.dropdown ul {
  font-size:14px;
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}
.nav-tabs {
  margin-top:20px;
}
.app-meta {
  padding:10px;
  background-color: #eeeeee;
  -moz-border-radius: 8px;
  -webkit-border-radius: 8px;
  border-radius: 8px;
}
.tab-content {
  margin-bottom:10px;
}
.app-meta .app-supplier {
  border-top:1px solid #ddd;
  padding-top:15px;
  margin-top:15px;
}
.app-supplier-data {
  overflow: hidden;
  padding-left: 10px;
}

.app-meta ul {
    padding: 15px 0;
}
.app-icon {
  line-height:0;
  overflow:hidden;
  margin-left:0;
  padding: 0;
  float: left;
  max-width: 80px;
}
.app-icon img {
    max-height:70px;
    width: 100%;
    margin: 0;
}

#environment_switcher {
  display: inline-block;
}
.panel-visible{
  display: block;
}

.columns{
  clear: both;
  width: 330px;
  padding: 0 0 20px 0;
  line-height: 22px;
}

.colleft{
  float: left;
  width: 150px;
  line-height: 22px;
}

.colright{
  float: right;
  width: 150px;
  line-height: 22px;
}
.app-icon.server {
  margin-right:20px;
}

.no-results {
  margin-bottom: 25px;
}
.app_requirements ul li {
  font-weight: bold;
}
.app_requirements ul li ul {
  margin-left: 30px;
}
.app_requirements ul li ul li {
  font-weight: normal;
}

/* Bootstrap 3 grid always applies 15px left and right padding on grid columns */
/* special class to override first and last column in row */
.col-row:first-child {
  padding-left:0;
}
.col-row:last-child {
  padding-right: 0;
}

/* LANDSCAPE PHONE TO SMALL DESKTOP & PORTRAIT TABLET */
/* Workarounds/hacks for filter forms at top of catalog page when fluid grid... */
/* changes layout due to device/width settings in Bootstrap 3's grid  */
@media (max-width: 767px) {
  .col-xs-12 .pull-right {
    float: none !important;
  }

  .col-xs-12.col-row {
    padding-left: 0;
  }

  .col-xs-12 .table_actions form {
    float: none;
    margin-left: 0;
  }

  .col-xs-12 .table_search {
    margin-left: -5px;
  }

  .col-xs-12 .table_search .form-group {
    float: left;
    margin-right: 15px;
  }

  .col-xs-12 .table_search .form-control-feedback {
    top: 0;
  }

  .app-list .app .app-details-link {
    float: left;
  }

  .app-list .app .app-actions {
    float: left;
  }
}

.modal-header img {
  margin-top: -7px;
}

.modal-dialog h3, span.wizard_title {
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
}

.selected-field {
  color: #428BCA;
  margin-left: -12px;
  font-weight: bold;
  font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
}

.steps_list_container {
  margin: 0 -15px;
  border-top: #d3d3d3 1px solid;
}

.steps_list {
  padding: 10px 20px 0 20px;
}

.steps_list li {
  padding-bottom: 7px;
}

.steps_list li.done {
  color: #c8c6c8;
}

button.wizard_cancel {
  margin-right: 10px;
}

h3 .wizard_title {
  margin-top: 15px;
  margin-bottom: 15px;
}

.steps_list li.active {
  color: #d93c27;
  font-weight: bold;
}
