{% load i18n %}
{% load url from future %}
{% block page_header %}
  <div class='page-header'>
      <h2>
          <a href=" {% url 'horizon:murano:environments:index' %}">
              {% blocktrans %}Environments{% endblocktrans %}
          </a> >
          <a href=" {% url 'horizon:murano:environments:services' environment_id %}?tab=environment_details__deployments">
             {% blocktrans%}Environment{% endblocktrans %} {{ environment_name|truncatechars:60 }}
          </a> >
          {% blocktrans%}Deployment at{% endblocktrans %} {{ deployment_start_time }}
      </h2>
  </div>
{% endblock %}
