{% load i18n %}
{% load url from future %}

{% trans "Quick Deploy" as label %}
<a href="{% url 'horizon:murano:catalog:quick_deploy' app.id %}" class="{{ class }}"><i class="fa fa-rocket"></i> {{ label }}</a>
