{% extends 'murano_base.html' %}
{% load i18n sizeformat %}
{% load static %}
{% load compress %}
{% block title %}{% trans "Component Details" %}{% endblock %}

{% block css %}
  {{ block.super }}
  {% compress css %}
    <link rel="stylesheet" href="{% static 'muranodashboard/css/reports.css' %}">
  {% endcompress %}
{% endblock %}

{% block page_header %}
    {% include "services/_detail_page_header.html" %}
{% endblock page_header %}

{% block main %}
<div class="row">
  <div class="col-sm-12">
  {{ tab_group.render }}
  </div>
</div>
{% endblock %}
