{% extends 'murano_base.html' %}
{% load i18n %}
{% block title %}{% trans "Import Package" %}{% endblock %}

{% block page_header %}
  {% include 'horizon/common/_page_header.html' with title=_('Import Package') %}
{% endblock page_header %}

{% block main %}
  {% include 'packages/_upload.html' %}
{% endblock %}
