{% load i18n %}
{% load url from future %}


<div class="status row detail app_license">
    <p>
        {% if application.license %}
            {{ application.license|linebreaksbr }}
        {% else %}
            {% trans 'No license' %}
        {% endif %}
    </p>

</div>
