{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% load static %}
{% load url from future %}

{% block form_id %}import_bundle{% endblock %}
{% block form_action %}{% url 'horizon:murano:packages:import_bundle' %}{% endblock %}

{% block modal_id %}upload_bundle_modal{% endblock %}
{% block modal-header %}{% trans 'Import Bundle' %}{% endblock %}

{% block modal-body %}
    <div class="left" >
    {{ wizard.management_form }}
    {% if wizard.form.forms %}
        {{ wizard.form.management_form }}
        {% for form in wizard.form.forms %}
            {{ form }}
        {% endfor %}
    {% else %}
        <fieldset>
        {% with form=wizard.form %}
            {% include "horizon/common/_form_fields.html" %}
        {% endwith %}
        </fieldset>
    {% endif %}
    </div>
    <div class="right">
        <div class='description-by_name'>
            <h3>{% trans 'Description:' %}</h3>
            <p>{% blocktrans %}<strong>Bundle Name: </strong>Bundle's full name.{% endblocktrans %}</p>
            <p>{% blocktrans %}The bundle is going to be installed from <a href="{{murano_repo_url}}" target="_blank">{{murano_repo_url}}</a> repository.{% endblocktrans %}</p>
        </div>
        <div class='description-by_url'>
            <h3>{% trans 'Description:' %}</h3>
            <p>{% blocktrans %}<strong>Bundle URL:</strong> HTTP/HTTPS URL of the bundle file.{% endblocktrans %}</p>
        </div>
        <div class='description-common'>
            <p>{% blocktrans %}<strong><i>Note: </i></strong>You'll have to configure each package installed from this bundle separately.
            <br>If packages depend upon other packages and/or require specific glance images, those are going to be installed with them from murano repository.
            {% endblocktrans %}</p>
        </div>
    </div>
{% endblock %}

{% block modal-footer %}
    {% if wizard.steps.next %}
        <input type='submit' class='btn btn-primary pull-right'  value='{% trans 'Next' %}'/>
    {% else %}
        <input type='submit' class='btn btn-primary pull-right'  value='{% trans 'Create' %}'/>
    {% endif %}
    <a href='{% url 'horizon:murano:packages:index' %}' class='btn btn-default secondary cancel close'>{% trans 'Cancel' %}</a>
{% endblock %}
