{% load i18n %}
{% load url from future %}

{% block page_header %}
  <div class='page-header'>
      <h1>
          {% blocktrans %}Application details: {{ name }}{% endblocktrans %}
      </h1>

      <h3>
          <a href=" {% url 'horizon:murano:catalog:index' %}">
              {% blocktrans %}Applications{% endblocktrans %}
          </a> > {{ name }}
      </h3>
  </div>
{% endblock %}
